@@ -177,6 +177,7 @@ public:
      *  @{ */
     std::shared_ptr<frontend::Connector>    the_connector() override;
     std::shared_ptr<frontend::Connector>    the_wayland_connector() override;
+    std::shared_ptr<frontend::Connector>    the_xwayland_connector() override;
     std::shared_ptr<frontend::Connector>    the_prompt_connector() override;
     std::shared_ptr<graphics::Display>      the_display() override;
     std::shared_ptr<compositor::Compositor> the_compositor() override;
@@ -381,6 +382,7 @@ protected:
 
     CachedPtr<frontend::Connector>   connector;
     CachedPtr<frontend::Connector>   wayland_connector;
+    CachedPtr<frontend::Connector>   xwayland_connector;
     CachedPtr<frontend::Connector>   prompt_connector;
 
     CachedPtr<input::InputReport> input_report;
@@ -448,11 +450,11 @@ protected:
     CachedPtr<scene::ApplicationNotRespondingDetector> application_not_responding_detector;
     CachedPtr<cookie::Authority> cookie_authority;
     CachedPtr<input::KeyMapper> key_mapper;
-    CachedPtr<ConsoleServices> console_services;
+    std::shared_ptr<ConsoleServices> console_services;
 
 private:
     std::shared_ptr<options::Configuration> const configuration_options;
-    std::shared_ptr<input::EventFilter> const default_filter;
+    std::shared_ptr<input::EventFilter> default_filter;
     CachedPtr<ObserverMultiplexer<graphics::DisplayConfigurationObserver>>
         display_configuration_observer_multiplexer;
     CachedPtr<ObserverMultiplexer<input::SeatObserver>>
