@@ -24,15 +24,24 @@
 #include <mir/main_loop.h>
 #include <mir/report_exception.h>
 #include <mir/options/option.h>
+#include <mir/options/configuration.h>
 
 #include <chrono>
 #include <mutex>
 #include <thread>
 
+
+namespace mo = mir::options;
+
 namespace
 {
 inline auto filename(std::string path) -> std::string
 {
+    // Remove the annoying ".bin" extension on Mir development binaries
+    auto const bin = path.rfind(".bin");
+    if (bin+4 == path.size())
+        path.erase(bin);
+
     return path.substr(path.rfind('/')+1);
 }
 }
@@ -40,7 +49,7 @@ inline auto filename(std::string path) -> std::string
 struct miral::MirRunner::Self
 {
     Self(int argc, char const* argv[], std::string const& config_file) :
-        argc(argc), argv(argv), config_file{config_file} {}
+        argc(argc), argv(argv), config_file{config_file}, display_config_file{filename(argv[0])+ ".display"} {}
 
     auto run_with(std::initializer_list<std::function<void(::mir::Server&)>> options) -> int;
     void launch_startup_applications(::mir::Server& server);
@@ -48,6 +57,7 @@ struct miral::MirRunner::Self
     int const argc;
     char const** const argv;
     std::string const config_file;
+    std::string const display_config_file;
 
     std::mutex mutex;
     std::function<void()> start_callback{[]{}};
@@ -137,7 +147,13 @@ void miral::MirRunner::Self::launch_startup_applications(::mir::Server& server)
                     if (app[0] == "gnome-terminal")
                         app.push_back("--app-id"),app.push_back("com.canonical.miral.Terminal");
 
-                    launch_app(app, wayland_display, mir_socket);
+                    mir::optional_value<std::string> x11_display;
+
+                    auto const options = server.get_options();
+                    if (options->is_set(mo::x11_display_opt))
+                        x11_display = std::string(":") + std::to_string(options->get<int>(mo::x11_display_opt));
+
+                    launch_app(app, wayland_display, mir_socket, x11_display);
 
                     if ((i = j) != end(value)) ++i;
                 }
@@ -168,7 +184,7 @@ try
         for (auto& option : options)
             option(*server);
 
-        server->add_stop_callback(stop_callback);
+        server->add_stop_callback(std::move(stop_callback));
 
         // Provide the command line and run the server
         server->set_command_line(argc, argv);
@@ -187,7 +203,7 @@ try
         // By enqueuing the notification code in the main loop, we are
         // ensuring that the server has really and fully started.
         auto const main_loop = server->the_main_loop();
-        main_loop->enqueue(this, start_callback);
+        main_loop->enqueue(this, std::move(start_callback));
     });
 
     server->run();
@@ -250,3 +266,13 @@ void miral::MirRunner::stop()
     }
 }
 
+auto miral::MirRunner::config_file() const -> std::string
+{
+    return self->config_file;
+}
+
+auto miral::MirRunner::display_config_file() const -> std::string
+{
+    return self->display_config_file;
+}
+
