@@ -82,6 +82,8 @@ public:
         {
             BOOST_THROW_EXCEPTION((std::system_error{errno, std::system_category(), "Failed to mmap() buffer"}));
         }
+
+        ::memset(map, 0, pitch_ * height);
     }
     ~DumbFb() noexcept(false)
     {
@@ -269,16 +271,21 @@ void mgek::EGLOutput::configure(size_t kms_mode_index)
     }
 
     EGLAttrib const crtc_filter[] = {
-        EGL_DRM_PLANE_EXT, static_cast<EGLAttrib>(plane_id),
+        EGL_DRM_CRTC_EXT, static_cast<EGLAttrib>(crtc_id),
         EGL_NONE};
     int found_layers{0};
     if (eglGetOutputLayersEXT(display, crtc_filter, &layer, 1, &found_layers) != EGL_TRUE)
     {
-        BOOST_THROW_EXCEPTION((mg::egl_error("Failed to find EGLOutputEXT corresponding to DRM CRTC")));
+        BOOST_THROW_EXCEPTION((
+            mg::egl_error(
+                std::string{"Failed to find EGLOutputEXT corresponding to DRM CRTC "} +
+                std::to_string(crtc_id))));
     }
     if (found_layers != 1)
     {
-        BOOST_THROW_EXCEPTION(std::runtime_error{"Failed to find EGLOutputEXT corresponding to DRM CRTC"});
+        BOOST_THROW_EXCEPTION(std::runtime_error{
+            std::string{"Failed to find EGLOutputEXT corresponding to DRM CRTC "} +
+            std::to_string(crtc_id)});
     }
 
     using_saved_crtc = false;
@@ -289,6 +296,19 @@ EGLOutputLayerEXT mgek::EGLOutput::output_layer() const
     return layer;
 }
 
+uint32_t mgek::EGLOutput::crtc_id() const
+{
+    EGLAttrib crtc_id;
+    if (eglQueryOutputLayerAttribEXT(display, layer, EGL_DRM_CRTC_EXT, &crtc_id)  != EGL_TRUE)
+    {
+        BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query DRM CRTC ID from EGLOutputLayer"));
+    } else if (crtc_id == 0)
+    {
+        BOOST_THROW_EXCEPTION((std::runtime_error{"EGLOutputLayer is associated with invalid CRTC?!"}));
+    }
+    return static_cast<uint32_t>(crtc_id);
+}
+
 void mgek::EGLOutput::clear_crtc()
 {
     using namespace std::string_literals;
