@@ -29,6 +29,14 @@
 #include "mir/graphics/egl_error.h"
 #include "mir/graphics/buffer_properties.h"
 #include "mir/graphics/buffer_ipc_message.h"
+#include "mir/raii.h"
+#include "mir/graphics/display.h"
+#include "mir/renderer/gl/context_source.h"
+#include "mir/renderer/gl/context.h"
+#include "mir/graphics/program_factory.h"
+#include "mir/graphics/program.h"
+#include "mir/executor.h"
+
 #include <boost/throw_exception.hpp>
 #include <boost/exception/errinfo_errno.hpp>
 
@@ -205,13 +213,43 @@ auto make_texture_binder(
         return std::make_unique<NativePixmapTextureBinder>(bo, egl_extensions);
 }
 
+std::unique_ptr<mir::renderer::gl::Context> context_for_output(mg::Display const& output)
+{
+    try
+    {
+        auto& context_source = dynamic_cast<mir::renderer::gl::ContextSource const&>(output);
+
+        /*
+         * We care about no part of this context's config; we will do no rendering with it.
+         * All we care is that we can allocate texture IDs and bind a texture, which is
+         * config independent.
+         *
+         * That's not *entirely* true; we also need it to be on the same device as we want
+         * to do the rendering on, and that GL must support all the extensions we care about,
+         * but since we don't yet support heterogeneous hybrid and implementing that will require
+         * broader interface changes it's a safe enough requirement for now.
+         */
+        return context_source.create_gl_context();
+    }
+    catch (std::bad_cast const& err)
+    {
+        std::throw_with_nested(
+            boost::enable_error_info(
+                std::runtime_error{"Output platform cannot provide a GL context"})
+                << boost::throw_function(__PRETTY_FUNCTION__)
+                << boost::throw_line(__LINE__)
+                << boost::throw_file(__FILE__));
+    }
+}
 }
 
 mgm::BufferAllocator::BufferAllocator(
+    mg::Display const& output,
     gbm_device* device,
     BypassOption bypass_option,
     mgm::BufferImportMethod const buffer_import_method)
-    : device(device),
+    : ctx{context_for_output(output)},
+      device(device),
       egl_extensions(std::make_shared<mg::EGLExtensions>()),
       bypass_option(buffer_import_method == mgm::BufferImportMethod::dma_buf ?
                         mgm::BypassOption::prohibited :
@@ -326,232 +364,235 @@ std::vector<MirPixelFormat> mgm::BufferAllocator::supported_pixel_formats()
 
 namespace
 {
-class WaylandBuffer :
-    public mir::graphics::BufferBasic,
-    public mir::graphics::NativeBufferBase,
-    public mir::renderer::gl::TextureSource
+GLuint get_tex_id()
+{
+    GLuint tex;
+    glGenTextures(1, &tex);
+    return tex;
+}
+
+geom::Size get_wl_buffer_size(wl_resource* buffer, mg::EGLExtensions::WaylandExtensions const& ext)
+{
+    EGLint width, height;
+
+    auto dpy = eglGetCurrentDisplay();
+    if (ext.eglQueryWaylandBufferWL(dpy, buffer, EGL_WIDTH, &width) == EGL_FALSE)
+    {
+        BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query WaylandAllocator buffer width"));
+    }
+    if (ext.eglQueryWaylandBufferWL(dpy, buffer, EGL_HEIGHT, &height) == EGL_FALSE)
+    {
+        BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query WaylandAllocator buffer height"));
+    }
+
+    return geom::Size{width, height};
+}
+
+mg::gl::Texture::Layout get_texture_layout(
+    wl_resource* resource,
+    mg::EGLExtensions::WaylandExtensions const& ext)
+{
+    EGLint inverted;
+    auto dpy = eglGetCurrentDisplay();
+
+    if (ext.eglQueryWaylandBufferWL(dpy, resource, EGL_WAYLAND_Y_INVERTED_WL, &inverted) == EGL_FALSE)
+    {
+        // EGL_WAYLAND_Y_INVERTED_WL is unsupported; the default is that the texture is in standard
+        // GL texture layout
+        return mg::gl::Texture::Layout::GL;
+    }
+    if (inverted)
+    {
+        // It has the standard y-decreases-with-row layout of GL textures
+        return mg::gl::Texture::Layout::GL;
+    }
+    else
+    {
+        // It has y-increases-with-row layout.
+        return mg::gl::Texture::Layout::TopRowFirst;
+    }
+}
+
+EGLint get_wl_egl_format(wl_resource* resource, mg::EGLExtensions::WaylandExtensions const& ext)
+{
+    EGLint format;
+    auto dpy = eglGetCurrentDisplay();
+
+    if (ext.eglQueryWaylandBufferWL(dpy, resource, EGL_TEXTURE_FORMAT, &format) == EGL_FALSE)
+    {
+        BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query Wayland buffer format"));
+    }
+    return format;
+}
+
+class WaylandTexBuffer :
+    public mg::BufferBasic,
+    public mg::NativeBufferBase,
+    public mg::gl::Texture
 {
 public:
-    static std::shared_ptr<mg::Buffer> mir_buffer_from_wl_buffer(
-        EGLDisplay dpy,
+    // Note: Must be called with a current EGL context
+    WaylandTexBuffer(
+        std::shared_ptr<mir::renderer::gl::Context> ctx,
         wl_resource* buffer,
-        std::shared_ptr<mg::EGLExtensions> const& extensions,
+        mg::EGLExtensions const& extensions,
         std::function<void()>&& on_consumed,
-        std::function<void()>&& on_release)
+        std::function<void()>&& on_release,
+        std::shared_ptr<mir::Executor> wayland_executor)
+        : ctx{std::move(ctx)},
+          tex{get_tex_id()},
+          on_consumed{std::move(on_consumed)},
+          on_release{std::move(on_release)},
+          size_{get_wl_buffer_size(buffer, *extensions.wayland)},
+          layout_{get_texture_layout(buffer, *extensions.wayland)},
+          egl_format{get_wl_egl_format(buffer, *extensions.wayland)},
+          wayland_executor{std::move(wayland_executor)}
     {
-        std::shared_ptr<WaylandBuffer> mir_buffer;
-        DestructionShim* shim;
-
-        if (auto notifier = wl_resource_get_destroy_listener(buffer, &on_buffer_destroyed))
+        if (egl_format != EGL_TEXTURE_RGB && egl_format != EGL_TEXTURE_RGBA)
         {
-            // We've already constructed a shim for this buffer, update it.
-            shim = wl_container_of(notifier, shim, destruction_listener);
+            BOOST_THROW_EXCEPTION((std::runtime_error{"YUV textures unimplemented"}));
+        }
+        eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
 
-            if (!(mir_buffer = shim->associated_buffer.lock()))
+        const EGLint image_attrs[] =
             {
-                /*
-                 * We've seen this wl_buffer before, but all the WaylandBuffers associated with it
-                 * have been destroyed.
-                 *
-                 * Recreate a new WaylandBuffer to track the new compositor lifetime.
-                 */
-                mir_buffer = std::shared_ptr<WaylandBuffer>{
-                    new WaylandBuffer{
-                        dpy,
-                        buffer,
-                        extensions,
-                        std::move(on_consumed),
-                        std::move(on_release)}};
-                shim->associated_buffer = mir_buffer;
-            }
-        }
-        else
-        {
-            mir_buffer = std::shared_ptr<WaylandBuffer>{
-                new WaylandBuffer{
-                    dpy,
-                    buffer,
-                    extensions,
-                    std::move(on_consumed),
-                    std::move(on_release)}};
-            shim = new DestructionShim;
-            shim->destruction_listener.notify = &on_buffer_destroyed;
-            shim->associated_buffer = mir_buffer;
+                EGL_IMAGE_PRESERVED_KHR, EGL_TRUE,
+                EGL_WAYLAND_PLANE_WL, 0,
+                EGL_NONE
+            };
 
-            wl_resource_add_destroy_listener(buffer, &shim->destruction_listener);
-        }
+        auto egl_image = extensions.eglCreateImageKHR(
+            eglGetCurrentDisplay(),
+            EGL_NO_CONTEXT,
+            EGL_WAYLAND_BUFFER_WL,
+            buffer,
+            image_attrs);
 
-        mir_buffer->buffer_mutex = shim->mutex;
-        return mir_buffer;
-    }
-
-    ~WaylandBuffer()
-    {
-        if (egl_image != EGL_NO_IMAGE_KHR)
-            extensions->eglDestroyImageKHR(dpy, egl_image);
-
-        std::lock_guard<std::mutex> lock{*buffer_mutex};
-        if (buffer)
-        {
-            on_release();
-        }
-    }
-
-    void gl_bind_to_texture() override
-    {
-        std::unique_lock<std::mutex> lock{*buffer_mutex};
-        if (buffer == nullptr)
-        {
-            mir::log_warning("WaylandBuffer::gl_bind_to_texture() called on a destroyed wl_buffer", this);
-            return;
-        }
         if (egl_image == EGL_NO_IMAGE_KHR)
-        {
-            eglBindAPI(MIR_SERVER_EGL_OPENGL_API);
+            BOOST_THROW_EXCEPTION(mg::egl_error("Failed to create EGLImage"));
 
-            const EGLint image_attrs[] =
-                {
-                    EGL_IMAGE_PRESERVED_KHR, EGL_TRUE,
-                    EGL_NONE
-                };
+        glBindTexture(GL_TEXTURE_2D, tex);
+        extensions.glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, egl_image);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-            egl_image = extensions->eglCreateImageKHR(
-                dpy,
-                EGL_NO_CONTEXT,
-                EGL_WAYLAND_BUFFER_WL,
-                buffer,
-                image_attrs);
-
-            if (egl_image == EGL_NO_IMAGE_KHR)
-                BOOST_THROW_EXCEPTION(mg::egl_error("Failed to create EGLImage"));
-        }
-        lock.unlock();
-
-        extensions->glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, egl_image);
+        // tex is now an EGLImage sibling, so we can free the EGLImage without
+        // freeing the backing data.
+        extensions.eglDestroyImageKHR(eglGetCurrentDisplay(), egl_image);
     }
 
-    void bind() override
+    ~WaylandTexBuffer()
     {
-        gl_bind_to_texture();
-    }
+        wayland_executor->spawn(
+            [context = ctx, tex = tex]()
+            {
+                context->make_current();
 
-    void secure_for_render() override
-    {
-        on_consumed();
+                glDeleteTextures(1, &tex);
+
+                context->release_current();
+            });
+
+        on_release();
     }
 
     std::shared_ptr<mir::graphics::NativeBuffer> native_buffer_handle() const override
     {
-        return nullptr;
+        return {nullptr};
     }
 
     mir::geometry::Size size() const override
     {
-        return mir::geometry::Size{width, height};
+        return size_;
     }
 
     MirPixelFormat pixel_format() const override
     {
-        return format;
+        /* TODO: These are lies, but the only piece of information external code uses
+         * out of the MirPixelFormat is whether or not the buffer has an alpha channel.
+         */
+        switch(egl_format)
+        {
+            case EGL_TEXTURE_RGB:
+                return mir_pixel_format_xrgb_8888;
+            case EGL_TEXTURE_RGBA:
+                return mir_pixel_format_argb_8888;
+            case EGL_TEXTURE_EXTERNAL_WL:
+                // Unspecified whether it has an alpha channel; say it does.
+                return mir_pixel_format_argb_8888;
+            case EGL_TEXTURE_Y_U_V_WL:
+            case EGL_TEXTURE_Y_UV_WL:
+                // These are just absolutely not RGB at all!
+                // But they're defined to not have an alpha channel, so xrgb it is!
+                return mir_pixel_format_xrgb_8888;
+            case EGL_TEXTURE_Y_XUXV_WL:
+                // This is a planar format, but *does* have alpha.
+                return mir_pixel_format_argb_8888;
+            default:
+                // We've covered all possibilities above
+                BOOST_THROW_EXCEPTION((std::logic_error{"Unexpected texture format!"}));
+        }
     }
 
-    mir::graphics::NativeBufferBase *native_buffer_base() override
+    NativeBufferBase* native_buffer_base() override
     {
         return this;
     }
 
+    mir::graphics::gl::Program const& shader(mir::graphics::gl::ProgramFactory& cache) const override
+    {
+        static std::unique_ptr<mg::gl::Program> shader;
+        if (!shader)
+        {
+            shader = cache.compile_fragment_shader(
+                "",
+                "uniform sampler2D tex;\n"
+                "vec4 sample_to_rgba(in vec2 texcoord)\n"
+                "{\n"
+                "    return texture2D(tex, texcoord);\n"
+                "}\n");
+        }
+        return *shader;
+    }
+
+    Layout layout() const override
+    {
+        return layout_;
+    }
+
+    void bind() override
+    {
+        glBindTexture(GL_TEXTURE_2D, tex);
+        on_consumed();
+        on_consumed = [](){};
+    }
+
+    void add_syncpoint() override
+    {
+    }
 private:
-    WaylandBuffer(
-        EGLDisplay dpy,
-        wl_resource* buffer,
-        std::shared_ptr<mg::EGLExtensions> const& extensions,
-        std::function<void()>&& on_consumed,
-        std::function<void()>&& on_release)
-        : buffer{buffer},
-        dpy{dpy},
-        egl_image{EGL_NO_IMAGE_KHR},
-        extensions{extensions},
-        on_consumed{std::move(on_consumed)},
-        on_release{std::move(on_release)}
-    {
-        if (extensions->wayland->eglQueryWaylandBufferWL(dpy, buffer, EGL_WIDTH, &width) == EGL_FALSE)
-        {
-            BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query WaylandAllocator buffer width"));
-        }
-        if (extensions->wayland->eglQueryWaylandBufferWL(dpy, buffer, EGL_HEIGHT, &height) == EGL_FALSE)
-        {
-            BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query WaylandAllocator buffer height"));
-        }
+    std::shared_ptr<mir::renderer::gl::Context> const ctx;
+    GLuint const tex;
 
-        EGLint texture_format;
-        if (!extensions->wayland->eglQueryWaylandBufferWL(dpy, buffer, EGL_TEXTURE_FORMAT, &texture_format))
-        {
-            BOOST_THROW_EXCEPTION(mg::egl_error("Failed to query WL buffer format"));
-        }
-
-        if (texture_format == EGL_TEXTURE_RGB)
-        {
-            format = mir_pixel_format_xrgb_8888;
-        }
-        else if (texture_format == EGL_TEXTURE_RGBA)
-        {
-            format = mir_pixel_format_argb_8888;
-        }
-        else
-        {
-            BOOST_THROW_EXCEPTION((std::invalid_argument{"YUV buffers are unimplemented"}));
-        }
-    }
-
-    static void on_buffer_destroyed(wl_listener* listener, void*)
-    {
-        static_assert(
-                std::is_standard_layout<DestructionShim>::value,
-                "DestructionShim must be Standard Layout for wl_container_of to be defined behaviour");
-
-        DestructionShim* shim;
-        shim = wl_container_of(listener, shim, destruction_listener);
-
-        {
-            std::lock_guard<std::mutex> lock{*shim->mutex};
-            if (auto mir_buffer = shim->associated_buffer.lock())
-            {
-                mir_buffer->buffer = nullptr;
-            }
-        }
-
-        delete shim;
-    }
-
-    struct DestructionShim
-    {
-        std::shared_ptr<std::mutex> const mutex = std::make_shared<std::mutex>();
-        std::weak_ptr<WaylandBuffer> associated_buffer;
-        wl_listener destruction_listener;
-    };
-
-    std::shared_ptr<std::mutex> buffer_mutex;
-    wl_resource* buffer;
-
-    EGLDisplay dpy;
-    EGLImageKHR egl_image;
-
-    EGLint width, height;
-    MirPixelFormat format;
-
-    std::shared_ptr<mg::EGLExtensions> const extensions;
-
-    std::function<void()> const on_consumed;
+    std::function<void()> on_consumed;
     std::function<void()> const on_release;
+
+    geom::Size const size_;
+    Layout const layout_;
+    EGLint const egl_format;
+
+    std::shared_ptr<mir::Executor> const wayland_executor;
 };
 }
 
-void mgm::BufferAllocator::bind_display(wl_display* display)
+void mgm::BufferAllocator::bind_display(wl_display* display, std::shared_ptr<Executor> wayland_executor)
 {
-    dpy = eglGetCurrentDisplay();
-
-    if (dpy == EGL_NO_DISPLAY)
-        BOOST_THROW_EXCEPTION((std::logic_error{"WaylandAllocator::bind_display called without an active EGL Display"}));
+    auto context_guard = mir::raii::paired_calls(
+        [this]() { ctx->make_current(); },
+        [this]() { ctx->release_current(); });
+    auto dpy = eglGetCurrentDisplay();
 
     if (!egl_extensions->wayland)
     {
@@ -566,6 +607,7 @@ void mgm::BufferAllocator::bind_display(wl_display* display)
     {
         mir::log_info("Bound WaylandAllocator display");
     }
+    this->wayland_executor = std::move(wayland_executor);
 }
 
 std::shared_ptr<mg::Buffer> mgm::BufferAllocator::buffer_from_resource(
@@ -573,10 +615,15 @@ std::shared_ptr<mg::Buffer> mgm::BufferAllocator::buffer_from_resource(
     std::function<void()>&& on_consumed,
     std::function<void()>&& on_release)
 {
-    return WaylandBuffer::mir_buffer_from_wl_buffer(
-        dpy,
+    auto context_guard = mir::raii::paired_calls(
+        [this]() { ctx->make_current(); },
+        [this]() { ctx->release_current(); });
+
+    return std::make_shared<WaylandTexBuffer>(
+        ctx,
         buffer,
-        egl_extensions,
+        *egl_extensions,
         std::move(on_consumed),
-        std::move(on_release));
+        std::move(on_release),
+        wayland_executor);
 }
