@@ -100,6 +100,20 @@ mgmh::DRMHelper::open_all_devices(
             continue;
         }
 
+        auto busid = std::unique_ptr<char, decltype(&drmFreeBusid)>{
+            drmGetBusid(tmp_fd), &drmFreeBusid
+        };
+
+        if (getenv("MIR_MESA_KMS_DISABLE_MODESET_PROBE") == nullptr)
+        {
+            if (auto modeset_error = -drmCheckModesettingSupported(busid.get()))
+            {
+                mir::log_info("Ignoring non-KMS DRM device %s", device.devnode());
+                error = modeset_error;
+                continue;
+            }
+        }
+
         // Can't use make_shared with the private constructor.
         opened_devices.push_back(
             std::shared_ptr<DRMHelper>{
@@ -312,7 +326,7 @@ mgm::GBMSurfaceUPtr mgmh::GBMHelper::create_scanout_surface(
     }
 
     auto surface_raw = gbm_surface_create(device, width, height,
-                                          GBM_BO_FORMAT_XRGB8888,
+                                          GBM_FORMAT_XRGB8888,
                                           format_flags);
 
     auto gbm_surface_deleter = [](gbm_surface *p) { if (p) gbm_surface_destroy(p); };
