@@ -125,11 +125,17 @@ mg::PlatformPriority probe_graphics_platform(
         mir::log_info("Unsupported: EGL platform does not support client extensions.");
         return mg::PlatformPriority::unsupported;
     }
-    if (strstr(client_extensions, "EGL_MESA_platform_gbm") == nullptr)
+    if (strstr(client_extensions, "EGL_KHR_platform_gbm") == nullptr)
     {
-        // No platform_gbm support, so we can't work.
-        mir::log_info("Unsupported: EGL platform does not support EGL_MESA_platform_gbm extension");
-        return mg::PlatformPriority::unsupported;
+        // Doesn't support the Khronos-standardised GBM platform…
+        mir::log_info("EGL platform does not support EGL_KHR_platform_gbm extension");
+        // …maybe we support the old pre-standardised Mesa GBM platform?
+        if (strstr(client_extensions, "EGL_MESA_platform_gbm") == nullptr)
+        {
+            mir::log_info(
+                "Unsupported: EGL platform supports neither EGL_KHR_platform_gbm nor EGL_MESA_platform_gbm");
+            return mg::PlatformPriority::unsupported;
+        }
     }
 
     // Check for suitability
@@ -156,6 +162,41 @@ mg::PlatformPriority probe_graphics_platform(
 
             if (tmp_fd != mir::Fd::invalid)
             {
+                // Check that the drm device is usable by setting the interface version we use (1.4)
+                drmSetVersion sv;
+                sv.drm_di_major = 1;
+                sv.drm_di_minor = 4;
+                sv.drm_dd_major = -1;     /* Don't care */
+                sv.drm_dd_minor = -1;     /* Don't care */
+
+                if (auto error = -drmSetInterfaceVersion(tmp_fd, &sv))
+                {
+                    throw std::system_error{
+                        error,
+                        std::system_category(),
+                        std::string{"Failed to set DRM interface version on device "} + device.devnode()};
+                }
+
+                /* Check if modesetting is supported on this DRM node
+                 * This must be done after drmSetInterfaceVersion() as, for Hysterical Raisins,
+                 * drmGetBusid() will return nullptr unless drmSetInterfaceVersion() has already been called
+                 */
+                auto const busid = std::unique_ptr<char, decltype(&drmFreeBusid)>{
+                    drmGetBusid(tmp_fd),
+                    &drmFreeBusid
+                };
+
+                if (getenv("MIR_MESA_KMS_DISABLE_MODESET_PROBE") == nullptr)
+                {
+                    if (auto err = -drmCheckModesettingSupported(busid.get()))
+                    {
+                        throw std::system_error{
+                            err,
+                            std::system_category(),
+                            std::string("Device ") + device.devnode() + " does not support KMS"};
+                    }
+                }
+
                 mgm::helpers::GBMHelper gbm_device{tmp_fd};
                 mgm::helpers::EGLHelper egl{MinimalGLConfig()};
 
