@@ -43,9 +43,10 @@ mgx::Platform::Platform(std::shared_ptr<::Display> const& conn,
     auth_factory = std::make_unique<mgm::DRMNativePlatformAuthFactory>(*drm);
 }
 
-mir::UniqueModulePtr<mg::GraphicBufferAllocator> mgx::Platform::create_buffer_allocator()
+mir::UniqueModulePtr<mg::GraphicBufferAllocator> mgx::Platform::create_buffer_allocator(
+    mg::Display const& output)
 {
-    return make_module_ptr<mgm::BufferAllocator>(gbm.device, mgm::BypassOption::prohibited, mgm::BufferImportMethod::dma_buf);
+    return make_module_ptr<mgm::BufferAllocator>(output, gbm.device, mgm::BypassOption::prohibited, mgm::BufferImportMethod::dma_buf);
 }
 
 mir::UniqueModulePtr<mg::Display> mgx::Platform::create_display(
