@@ -29,6 +29,8 @@
 
 namespace mir
 {
+class Executor;
+
 namespace input
 {
 class InputDeviceHub;
@@ -48,28 +50,48 @@ namespace frontend
 class WlCompositor;
 class WlSubcompositor;
 class WlApplication;
-class WlShell;
-class XdgShellV6;
 class WlSeat;
 class OutputManager;
 
 class Shell;
-class DisplayChanger;
+class MirDisplay;
 class SessionAuthorizer;
 class DataDeviceManager;
 
+class WaylandExtensions
+{
+public:
+    WaylandExtensions() = default;
+    virtual ~WaylandExtensions() = default;
+    WaylandExtensions(WaylandExtensions const&) = delete;
+    WaylandExtensions& operator=(WaylandExtensions const&) = delete;
+
+    void init(wl_display* display, std::shared_ptr<Shell> const& shell, WlSeat* seat, OutputManager* const output_manager);
+
+    auto get_extension(std::string const& name) const -> std::shared_ptr<void>;
+
+protected:
+
+    virtual void add_extension(std::string const name, std::shared_ptr<void> implementation);
+    virtual void custom_extensions(wl_display* display, std::shared_ptr<Shell> const& shell, WlSeat* seat, OutputManager* const output_manager);
+
+private:
+    std::unordered_map<std::string, std::shared_ptr<void>> extension_protocols;
+};
+
 class WaylandConnector : public Connector
 {
 public:
     WaylandConnector(
         optional_value<std::string> const& display_name,
         std::shared_ptr<Shell> const& shell,
-        DisplayChanger& display_config,
+        std::shared_ptr<MirDisplay> const& display_config,
         std::shared_ptr<input::InputDeviceHub> const& input_hub,
         std::shared_ptr<input::Seat> const& seat,
         std::shared_ptr<graphics::GraphicBufferAllocator> const& allocator,
         std::shared_ptr<SessionAuthorizer> const& session_authorizer,
-        bool arw_socket);
+        bool arw_socket,
+        std::unique_ptr<WaylandExtensions> extensions);
 
     ~WaylandConnector() override;
 
@@ -85,6 +107,9 @@ public:
 
     auto socket_name() const -> optional_value<std::string> override;
 
+    auto get_extension(std::string const& name) const -> std::shared_ptr<void>;
+    auto get_wl_display() const -> wl_display*;
+
 private:
     std::unique_ptr<wl_display, void(*)(wl_display*)> const display;
     mir::Fd const pause_signal;
@@ -92,10 +117,10 @@ private:
     std::unique_ptr<WlSubcompositor> subcompositor_global;
     std::unique_ptr<WlSeat> seat_global;
     std::unique_ptr<OutputManager> output_manager;
-    std::shared_ptr<graphics::WaylandAllocator> const allocator;
-    std::unique_ptr<WlShell> shell_global;
     std::unique_ptr<DataDeviceManager> data_device_manager_global;
-    std::unique_ptr<XdgShellV6> xdg_shell_global;
+    std::shared_ptr<Executor> const executor;
+    std::shared_ptr<graphics::WaylandAllocator> const allocator;
+    std::unique_ptr<WaylandExtensions> const extensions;
     std::thread dispatch_thread;
     wl_event_source* pause_source;
     std::string wayland_display;
@@ -103,6 +128,9 @@ private:
     // Only accessed on event loop
     std::unordered_map<int, std::function<void(std::shared_ptr<Session> const& session)>> mutable connect_handlers;
 };
+
+auto create_wl_shell(wl_display* display, std::shared_ptr<Shell> const& shell, WlSeat* seat, OutputManager* const output_manager)
+    -> std::shared_ptr<void>;
 }
 }
 
