@@ -22,10 +22,12 @@
 #include "pointer.h"
 #include "touch.h"
 #include "xdg_shell_v6.h"
+#include "xdg_shell_stable.h"
 #include "generated/wayland-client.h"
 #include "generated/xdg-shell-unstable-v6-client.h"
 #include "generated/xdg-shell-client.h"
 
+#include "linux/input.h"
 #include <boost/throw_exception.hpp>
 #include <stdexcept>
 #include <memory>
@@ -70,6 +72,20 @@ public:
             wl_fixed_from_int(dy));
     }
 
+    void button_down(int button)
+    {
+        wlcs_pointer_button_down(
+            pointer.get(),
+            button);
+    }
+
+    void button_up(int button)
+    {
+        wlcs_pointer_button_up(
+            pointer.get(),
+            button);
+    }
+
 private:
     std::unique_ptr<WlcsPointer, decltype(&wlcs_destroy_pointer)> const pointer;
 };
@@ -92,6 +108,37 @@ void wlcs::Pointer::move_by(int dx, int dy)
     impl->move_by(dx, dy);
 }
 
+void wlcs::Pointer::button_down(int button)
+{
+    impl->button_down(button);
+}
+
+void wlcs::Pointer::button_up(int button)
+{
+    impl->button_up(button);
+}
+
+void wlcs::Pointer::click(int button)
+{
+    button_down(button);
+    button_up(button);
+}
+
+void wlcs::Pointer::left_button_down()
+{
+    button_down(BTN_LEFT);
+}
+
+void wlcs::Pointer::left_button_up()
+{
+    button_up(BTN_LEFT);
+}
+
+void wlcs::Pointer::left_click()
+{
+    click(BTN_LEFT);
+}
+
 class wlcs::Touch::Impl
 {
 public:
@@ -435,6 +482,30 @@ public:
         return surface;
     }
 
+    Surface create_xdg_shell_stable_surface(Client& client, int width, int height)
+    {
+        Surface surface{client};
+
+        auto xdg_surface = std::make_shared<XdgSurfaceStable>(client, surface);
+        auto xdg_toplevel = std::make_shared<XdgToplevelStable>(*xdg_surface);
+
+        run_on_destruction([xdg_surface, xdg_toplevel]() mutable
+            {
+                xdg_surface.reset();
+                xdg_toplevel.reset();
+            });
+
+        wl_surface_commit(surface);
+
+        surface.attach_buffer(width, height);
+
+        bool surface_rendered{false};
+        surface.add_frame_callback([&surface_rendered](auto) { surface_rendered = true; });
+        wl_surface_commit(surface);
+        dispatch_until([&surface_rendered]() { return surface_rendered; });
+        return surface;
+    }
+
     wl_shell* the_shell() const
     {
         return shell;
@@ -490,6 +561,10 @@ public:
     {
         motion_notifiers.push_back(on_motion);
     }
+    void add_pointer_button_notification(PointerButtonNotifier const& on_button)
+    {
+        button_notifiers.push_back(on_button);
+    }
 
     void* acquire_interface(std::string const& interface, wl_interface const* to_bind, uint32_t version)
     {
@@ -639,6 +714,30 @@ private:
         }
     }
 
+    static void pointer_button(
+        void *ctx,
+        wl_pointer* /*wl_pointer*/,
+        uint32_t serial,
+        uint32_t /*time*/,
+        uint32_t button,
+        uint32_t state)
+    {
+        auto me = static_cast<Impl*>(ctx);
+
+        std::vector<decltype(button_notifiers)::const_iterator> to_remove;
+        for (auto notifier = me->button_notifiers.begin(); notifier != me->button_notifiers.end(); ++notifier)
+        {
+            if (!(*notifier)(serial, button, state == WL_POINTER_BUTTON_STATE_PRESSED))
+            {
+                to_remove.push_back(notifier);
+            }
+        }
+        for (auto removed : to_remove)
+        {
+            me->button_notifiers.erase(removed);
+        }
+    }
+
     static void pointer_frame(void* /*ctx*/, wl_pointer* /*pointer*/)
     {
     }
@@ -647,7 +746,7 @@ private:
         &Impl::pointer_enter,
         &Impl::pointer_leave,
         &Impl::pointer_motion,
-        nullptr,    // button
+        &Impl::pointer_button,
         nullptr,    // axis
         &Impl::pointer_frame,    // frame
         nullptr,    // axis_source
@@ -837,6 +936,7 @@ private:
     std::vector<PointerEnterNotifier> enter_notifiers;
     std::vector<PointerLeaveNotifier> leave_notifiers;
     std::vector<PointerMotionNotifier> motion_notifiers;
+    std::vector<PointerButtonNotifier> button_notifiers;
 };
 
 constexpr wl_pointer_listener wlcs::Client::Impl::pointer_listener;
@@ -901,6 +1001,11 @@ wlcs::Surface wlcs::Client::create_xdg_shell_v6_surface(int width, int height)
     return impl->create_xdg_shell_v6_surface(*this, width, height);
 }
 
+wlcs::Surface wlcs::Client::create_xdg_shell_stable_surface(int width, int height)
+{
+    return impl->create_xdg_shell_stable_surface(*this, width, height);
+}
+
 wlcs::Surface wlcs::Client::create_visible_surface(int width, int height)
 {
     return impl->create_wl_shell_surface(*this, width, height);
@@ -956,6 +1061,11 @@ void wlcs::Client::add_pointer_motion_notification(PointerMotionNotifier const&
     impl->add_pointer_motion_notification(on_motion);
 }
 
+void wlcs::Client::add_pointer_button_notification(PointerButtonNotifier const& on_button)
+{
+    impl->add_pointer_button_notification(on_button);
+}
+
 void wlcs::Client::dispatch_until(std::function<bool()> const& predicate)
 {
     impl->dispatch_until(predicate);
