@@ -19,6 +19,8 @@
 #ifndef MIR_TEST_DOUBLES_MOCK_DRM_H_
 #define MIR_TEST_DOUBLES_MOCK_DRM_H_
 
+#include "mir_test_framework/open_wrapper.h"
+
 #include <gmock/gmock.h>
 
 #include <xf86drm.h>
@@ -67,6 +69,7 @@ public:
                                        uint32_t clock, uint16_t htotal, uint16_t vtotal,
                                        ModePreference preferred);
 
+    bool drm_setversion_called{false};
 private:
     int pipe_fds[2];
 
@@ -148,6 +151,7 @@ public:
     MOCK_METHOD1(drmGetBusid, char* (int fd));
     MOCK_METHOD1(drmFreeBusid, void (const char*));
     MOCK_METHOD1(drmGetDeviceNameFromFd, char*(int fd));
+    MOCK_METHOD1(drmGetPrimaryDeviceNameFromFd, char*(int fd));
 
     MOCK_METHOD6(drmModeCrtcGetGamma, int(int fd, uint32_t crtc_id, uint32_t size,
                                           uint16_t* red, uint16_t* green, uint16_t* blue));
@@ -157,6 +161,7 @@ public:
     MOCK_METHOD1(drmGetVersion, drmVersionPtr(int));
     MOCK_METHOD1(drmFreeVersion, void(drmVersionPtr));
 
+    MOCK_METHOD1(drmCheckModesettingSupported, int(char const*));
 
     void add_crtc(
         char const* device,
@@ -182,6 +187,7 @@ public:
     void reset(char const* device);
 
     void generate_event_on(char const* device);
+    void consume_event_on(char const* device);
 
     class IsFdOfDeviceMatcher;
     friend class IsFdOfDeviceMatcher;
@@ -190,6 +196,7 @@ private:
     std::unordered_map<std::string, FakeDRMResources> fake_drms;
     std::unordered_map<int, FakeDRMResources&> fd_to_drm;
     drmModeObjectProperties empty_object_props;
+    mir_test_framework::OpenHandlerHandle open_interposer;
 };
 
 testing::Matcher<int> IsFdOfDevice(char const* device);
