@@ -18,6 +18,7 @@
  */
 
 #include "mir/test/doubles/mock_drm.h"
+#include "mir_test_framework/open_wrapper.h"
 #include "mir/geometry/size.h"
 #include <gtest/gtest.h>
 
@@ -232,6 +233,16 @@ drmModeModeInfo mtd::FakeDRMResources::create_mode(uint16_t hdisplay, uint16_t v
 }
 
 mtd::MockDRM::MockDRM()
+    : open_interposer{mir_test_framework::add_open_handler(
+        [this](char const* path, int flags, mode_t mode) -> std::experimental::optional<int>
+        {
+            char const* const drm_prefix = "/dev/dri/";
+            if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
+            {
+                return this->open(path, flags, mode);
+            }
+            return {};
+        })}
 {
     using namespace testing;
     assert(global_mock == NULL && "Only one mock object per process is allowed");
@@ -297,13 +308,38 @@ mtd::MockDRM::MockDRM()
         .WillByDefault(Return(&empty_object_props));
 
     ON_CALL(*this, drmSetInterfaceVersion(_, _))
-    .WillByDefault(Return(0));
+        .WillByDefault(
+            Invoke(
+                [this](auto fd, auto version)
+                {
+                    if (version->drm_di_major != 1 || version->drm_di_minor != 4)
+                    {
+                        ADD_FAILURE() << "We should only ever request DRM version 1.4";
+                        errno = EINVAL;
+                        return -1;
+                    }
+
+                    fd_to_drm.at(fd).drm_setversion_called = true;
+                    return 0;
+                }));
 
     ON_CALL(*this, drmGetBusid(_))
-    .WillByDefault(WithoutArgs(Invoke([]{ return static_cast<char*>(malloc(10)); })));
+        .WillByDefault(
+            Invoke(
+                [this](auto fd) -> char*
+                {
+                    if (!fd_to_drm.at(fd).drm_setversion_called)
+                    {
+                        return nullptr;
+                    }
+                    return static_cast<char*>(malloc(10));
+                }));
 
     ON_CALL(*this, drmFreeBusid(_))
-    .WillByDefault(WithArg<0>(Invoke([&](const char* busid){ free(const_cast<char*>(busid)); })));
+        .WillByDefault(WithArg<0>(Invoke([&](const char* busid) { free(const_cast<char*>(busid)); })));
+
+    ON_CALL(*this, drmGetPrimaryDeviceNameFromFd(_))
+        .WillByDefault(InvokeWithoutArgs([]() { return strdup("/dev/dri/card0"); }));
 
     static drmVersion const version{
         1,
@@ -318,6 +354,11 @@ mtd::MockDRM::MockDRM()
     };
     ON_CALL(*this, drmGetVersion(_))
         .WillByDefault(Return(const_cast<drmVersionPtr>(&version)));
+
+    ON_CALL(*this, drmCheckModesettingSupported(NotNull()))
+        .WillByDefault(Return(0));
+    ON_CALL(*this, drmCheckModesettingSupported(IsNull()))
+        .WillByDefault(Return(-EINVAL));
 }
 
 mtd::MockDRM::~MockDRM() noexcept
@@ -360,6 +401,18 @@ void mtd::MockDRM::generate_event_on(char const *device)
     }
 }
 
+void mtd::MockDRM::consume_event_on(char const* device)
+{
+    auto const fd = fake_drms[device].fd();
+
+    char consume_buf;
+    if (::read(fd, &consume_buf, 1) != 1)
+    {
+        BOOST_THROW_EXCEPTION(
+            std::system_error(errno, std::system_category(), "Failed to consume fake DRM event"));
+    }
+}
+
 void mtd::MockDRM::add_connector(
     char const *device,
     uint32_t connector_id,
@@ -676,57 +729,12 @@ char* drmGetDeviceNameFromFd(int fd)
     return global_mock->drmGetDeviceNameFromFd(fd);
 }
 
-// We need to wrap open as we sometimes open() the DRM device
-// We need to explicitly mark this as C because we don't match the
-// libc header; we only care about the three-parameter version
-extern "C"
+char* drmGetPrimaryDeviceNameFromFd(int fd)
 {
-int open(char const* path, int flags, mode_t mode)
-{
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open) = dlsym(RTLD_NEXT, "open");
-
-    return (*real_open)(path, flags, mode);
+    return global_mock->drmGetPrimaryDeviceNameFromFd(fd);
 }
 
-int open64(char const* path, int flags, mode_t mode)
+int drmCheckModesettingSupported(char const* busid)
 {
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open64)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open64) = dlsym(RTLD_NEXT, "open64");
-
-    return (*real_open64)(path, flags, mode);
-}
-
-int __open(char const* path, int flags, mode_t mode)
-{
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open) = dlsym(RTLD_NEXT, "__open");
-
-    return (*real_open)(path, flags, mode);
-}
-
-int __open64(char const* path, int flags, mode_t mode)
-{
-    char const* drm_prefix = "/dev/dri/";
-    if (!strncmp(path, drm_prefix, strlen(drm_prefix)))
-        return global_mock->open(path, flags, mode);
-
-    int (*real_open64)(char const *path, int flags, mode_t mode);
-    *(void **)(&real_open64) = dlsym(RTLD_NEXT, "__open64");
-
-    return (*real_open64)(path, flags, mode);
-}
-
+    return global_mock->drmCheckModesettingSupported(busid);
 }
