@@ -114,6 +114,7 @@ public:
     void stop() override {}
 
     int client_socket_fd() const override { return 0; }
+    auto socket_name() const -> mir::optional_value<std::string> override { return {}; }
 
     MOCK_CONST_METHOD1(client_socket_fd, int (std::function<void(std::shared_ptr<mf::Session> const&)> const&));
 };
@@ -376,9 +377,9 @@ struct SessionMediator : public ::testing::Test
             {
             }
 
-            void handle_event(MirEvent const &e) override
+            void handle_event(mir::EventUPtr&& event) override
             {
-                wrapped->handle_event(e);
+                wrapped->handle_event(move(event));
             }
 
             void handle_lifecycle_event(MirLifecycleState state) override
